"""
Install OpenERP on a new (by default) database.
"""
import os
import sys

import common

# TODO turn template1 in a parameter
# This should be exposed from openerp (currently in
# openerp/service/web_services.py).
def create_database(database_name):
    import openerp
    db = openerp.sql_db.db_connect('template1')
    cr = db.cursor() # TODO `with db as cr:`
    try:
        cr.autocommit(True)
        cr.execute("""CREATE DATABASE "%s"
            ENCODING 'unicode' TEMPLATE "template1" """ \
            % (database_name,))
    finally:
        cr.close()

def run(args):
    assert args.database
    assert not (args.module and args.all_modules)

    import openerp

    config = openerp.tools.config
    config['log_handler'] = [':CRITICAL']
    if args.addons:
        args.addons = args.addons.split(':')
    else:
        args.addons = []
    config['addons_path'] = ','.join(args.addons)
    openerp.netsvc.init_logger()

    if args.all_modules:
        module_names = common.get_addons_from_paths(args.addons, args.exclude)
        config['init'] = dict.fromkeys(module_names, 1)

    if args.module:
        module_names = args.module
        config['init'] = dict.fromkeys(module_names, 1)

    if not args.no_create:
        create_database(args.database)

    # Install the import hook, to import openerp.addons.<module>.
    openerp.modules.module.initialize_sys_path()
    openerp.modules.loading.open_openerp_namespace()

    openerp.modules.registry.RegistryManager.get(
        args.database, update_module=True, pooljobs=False)

def add_parser(subparsers):
    parser = subparsers.add_parser('initialize',
        description='Create and initialize a new OpenERP database.')
    parser.add_argument('-d', '--database', metavar='DATABASE',
        **common.required_or_default('DATABASE', 'the database to create'))
    common.add_addons_argument(parser)
    parser.add_argument('--module', metavar='MODULE', action='append',
        help='specify a module to install'
        ' (this option can be repeated)')
    parser.add_argument('--all-modules', action='store_true',
        help='install all visible modules (not compatible with --module)')
    parser.add_argument('--no-create', action='store_true',
        help='do not create the database, only initialize it')
    parser.add_argument('--exclude', metavar='MODULE', action='append',
        help='exclude a module from installation'
        ' (this option can be repeated)')

    parser.set_defaults(run=run)
